# BookQueue Movable Type Plugin
# http://jacob.hesch.cc/software/mt-plugins/BookQueue/
#
# $Id: BookQueue.pm,v 1.1 2004/12/06 07:12:19 jlh Exp $
#
# Copyright (C) 2003 Jacob L Hesch
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

package MTPlugins::BookQueue;
use strict;

use vars qw( $VERSION );
$VERSION = 1.01;

use MT::PluginData;
use MT::Util qw( format_ts offset_time_list );

use MT::App::CMS;
@MTPlugins::BookQueue::ISA = qw( MT::App::CMS );

# NOTE: any mention here of an `entry_id' refers to a regular MT blog
# entry that may be associated with a BookQueue entry (BookQueue
# entries themselves do not have entry_id's)

sub init
{
    my $app = shift;
    $app->SUPER::init(@_) or return;
    $app->add_methods(
            'view' => \&show_queue,
            'save' => \&save_entry,
                      );
    $app->{default_mode}   = 'view';
    $app->{template_dir}   = 'cms';
    $app->{user_class}     = 'MT::Author';
    $app->{requires_login} = 1;
    $app;
}

sub save_entry
{
    my $app = shift;
    my $q = $app->{query};

    my $isbn        = $q->param('isbn');
    my $key         = $q->param('key');
    my $modified_on = $q->param('modified_on');
    my $mod_on_ovrd = $q->param('mod_on_ovrd'); # override modified_on
    my $entry_id    = $q->param('entry_id');
    my $blog_id     = $q->param('blog_id');
    my $status      = $q->param('status')  || 'unread';

    # this section copied from MT::App::CMS (slightly modified)
    # Copyright 2001-2003 Six Apart. This code cannot be redistributed without
    # permission from www.movabletype.org.
    require MT::Blog;
    my $blog = MT::Blog->load($blog_id);
    my @ts = offset_time_list(time, $blog);
    my $ts = sprintf "%04d%02d%02d%02d%02d%02d",
        $ts[5]+1900, $ts[4]+1, @ts[3,2,1,0];
    if ($q->param('mod_on_ovrd')) {
        unless ($modified_on =~
            m!^(\d{4})-(\d{2})-(\d{2})\s+(\d{2}):(\d{2})(?::(\d{2}))?$!) {
            return $app->error($app->translate(
                "Invalid date '[_1]'; last modified on date must be in the " .
                "format YYYY-MM-DD HH:MM:SS.", $modified_on));
        }
        my $s = $6 || 0;
        $modified_on = sprintf "%04d%02d%02d%02d%02d%02d", $1, $2, $3, $4, $5, $s;
    }
    else {
        $modified_on = $ts;
    }
    # end section copied from MT::App::CMS

    my ($data);
    unless ($key && ($data = MT::PluginData->load({ plugin => 'BookQueue',
                                                    key    => $key }))) {
        $data = MT::PluginData->new;
        $data->plugin('BookQueue');
        $data->key($ts);
    }

    if ($q->param('delete') && $data) {
        $data->remove or return
            $app->error($app->translate("Error: [_1]", $data->errstr));
        return show_queue($app);
    }

    $isbn = _decue($1)    if $isbn =~ /^\.[^.]+\.[^.]+\.([^.]+)/;
    $isbn = _ean2isbn($1) if $isbn =~ /^978(\d{9})\d$/;
    return $app->error($app->translate("Invalid ISBN: [_1]", $isbn))
        unless $isbn =~ /^\d{9}[\dx]$/i;

    $data->data({
        isbn        => $isbn,
        status      => $status,
        modified_on => $modified_on,
        mod_on_ovrd => $mod_on_ovrd,
        blog_id     => $blog_id,
        entry_id    => $entry_id,
    });
    $data->save or
        return $app->error($app->translate("Error: [_1]", $data->errstr));

    $q->param('rebuild', 1);
    show_queue($app);
}

sub show_queue
{
    my $app = shift;
    my $q = $app->{query};

    my $blog_id = $q->param('blog_id') || undef;
    my $limit   = $q->param('limit')   || 10;
    my $offset  = $q->param('offset')  || 0;
    my $show    = $q->param('show')    || 'all';

    my $dev_token = _read_dev_token($app);

    return $app->_select_blog() unless defined $blog_id;

    my $blog = MT::Blog->load($blog_id);

    # since the `blog_id' and `status' data fields of our MTPluginData
    # object are not real columns in the datastore, we cannot use the
    # `limit' and `offset' search parameters of MT::Object's load() and
    # load_iter() methods. so we must load *all* the plugin's entries
    # and do it ourselves...

    my $iter = MT::PluginData->load_iter({ plugin    => 'BookQueue' },
                                         { sort      => 'key',
                                           direction => 'descend'   });

    my $i         = 0; # loop iteration counter
    my $n_entries = 0; # the number of entries displayed on this page
    my $count     = 0; # the total number of (unpaginated) entries
    my @entry_data;

    while (my $entry = $iter->()) {
        next unless $blog_id == $entry->data->{blog_id};
        next unless $show eq 'all' or $show eq $entry->data->{status};
        $count++;
        next unless ++$i > $offset;   # skip over previous pages' entries.
        next if $n_entries == $limit; # we've added all the entries for this
                                      # page, but keep looping to count total

        my $mod_on_display = format_ts("%Y-%m-%d %H:%M:%S",
                                       $entry->data->{modified_on},
                                       $blog);
        my $row = {
            blog_id     => $blog_id,
            show        => $show,
            limit       => $limit,
            offset      => $offset,
            key         => $entry->key,
            isbn        => $entry->data->{isbn},
            entry_id    => $entry->data->{entry_id},
            mod_on_ovrd => $entry->data->{mod_on_ovrd},
            modified_on => $mod_on_display,
#            script_url  => $app->_bq_uri,
            entry_odd   => $n_entries % 2 ? 1 : 0,
            amzn_info   => _fetch_amazon_info($entry->data->{isbn},
                                              $dev_token),
        };
        my @status_data;
        for (qw( unread reading read )) {
            push @status_data, { status => $_ };
            $status_data[-1]{checked} = 1 if $entry->data->{status} eq $_;
        }
        $row->{status_loop} = \@status_data;
        push @entry_data, $row;
        $n_entries++;
    }

    my $param = {
        entry_loop => \@entry_data,
        empty      => !$n_entries,
        show       => $show,
        blog_id    => $blog_id,
        blog_name  => $blog->name,
        rebuild    => $q->param('rebuild') ? 1 : 0,
    };
    SWITCH: {
        $param->{show_all}     = 1, last SWITCH if $show eq 'all';
        $param->{show_unread}  = 1, last SWITCH if $show eq 'unread';
        $param->{show_reading} = 1, last SWITCH if $show eq 'reading';
        $param->{show_read}    = 1, last SWITCH if $show eq 'read';
    }

    # paginate
    $param->{limit}    = $limit;
    $param->{paginate} = 0;
    if ($offset > 0) {
        $param->{prev}        = 1;
        my $prev_offset       = $offset - $limit;
        $prev_offset          = 0 if $prev_offset < 0;
        $param->{prev_offset} = $prev_offset;
        $param->{paginate}    = 1;
    }
    if ($n_entries > 0 && $limit + $offset < $count) {
        $param->{next}        = 1;
        $param->{next_offset} = $offset + $limit;
        $param->{paginate}    = 1;
    }

    my @limit_data;
    for (1, 5, 10, 20, 50, 100) {
        push @limit_data, { limit       => $_,
                            limit_label => $_ };
        $limit_data[-1]{limit_selected} = 1 if $limit == $_;
    }
    $param->{limit_loop} = \@limit_data;

#    $app->{template_dir} = 'cms';
#    $app->build_page('bookqueue.tmpl', $param);

    $param->{bq_script_url} = $app->_bq_uri;
    my $tmpl_data = join '', <DATA>;
    require HTML::Template;
    my $tmpl = HTML::Template->new(scalarref => \$tmpl_data);
    for my $key (keys %$param) {
        $tmpl->param($key, $param->{$key});
    }
    $tmpl->output;
}

sub _select_blog
{
    my $app = shift;
    my ($param);

    $param->{bq_script_url} = $app->_bq_uri;
    $app->build_page('bookqueue-tmpl/select_blog.tmpl', $param);
}

# in order to play friendly with MT's cms templates, $app->uri must
# return the main cms uri (mt.cgi), so we override the method to do
# just that (and provide our own for the bookqueue uri)
sub uri
{
    my $app = shift;
    my $script = $app->script;
    $script =~ s/-bookqueue//;
    $app->path . $script;
}

sub _bq_uri { $_[0]->SUPER::uri }

sub _read_dev_token
{
    # code taken from MTAmazon
    # Copyright (c) 2002 Kalsey Consulting Group
    my $app = shift;
    require MT::ConfigMgr;
    my $cfg = MT::ConfigMgr->instance;
    my $path = $cfg->PluginPath;
    my $cfgfile = File::Spec->catfile($path,'.mtamazon.config');
    open(FH, $cfgfile) or return 0;
    my $dev_token;
    while (<FH>) {
        chomp;
        next if !/\S/ || /^#/;
        my($var, $val) = $_ =~ /^\s*(\S+)\s+(.+)$/;
        $var = lc($var);
        $val =~ s/\s*$//;
        $dev_token = $val, last if $var eq 'devtoken';
    }
    close(FH);
    $dev_token;
    # end MTAmazon code
}

sub _fetch_amazon_info
{
    my ($isbn, $dev_token) = @_;

    eval ("use XML::Simple");
    return "XML::Simple not installed" if ($@);

    eval ("use LWP::Simple");
    return "LWP::Simple not installed" if ($@);

    return "DevToken not found--<br />is MTAmazon installed<br />and working?"
        unless $dev_token;

    return "error retreiving<br />Amazon info" unless
        my $xml = get("http://xml.amazon.com/onca/xml2?t=jacobheschcc-20&dev-t=$dev_token&AsinSearch=$isbn&type=lite&f=xml");

    my $item   = XMLin($xml)->{Details};
    my $url    = $item->{url};
    my $img    = $item->{ImageUrlSmall};
    my $title  = $item->{ProductName};
    my $author = ref $item->{Authors}->{Author} eq "ARRAY"
        ? join(", ", @{ $item->{Authors}->{Author} })
        : $item->{Authors}->{Author};
    return "Amazon error--<br />invalid ISBN?" unless $url && $img;
    $title  ||= '[title unavailable]';
    $author ||= '[author unavailable]';

    qq!<a href="$url"><img src="$img" alt="$title by $author" border="0" /></a>!;
}

sub _ean2isbn
{
    # algorithm: http://www.bisg.org/booklanean.htm
    my @digits = split //, $_[0];
    my $sum = 0;
    $sum += $digits[$_] * (10 - $_) for (0 .. $#digits);
    my $check = 11 - $sum % 11;
    $check =  0  if $check == 11;
    $check = 'X' if $check == 10;
    join '', (@digits, $check);
}

sub _decue
{
    # tatooable decue code compliments of Larry Wall
    # http://www.beau.lib.la.us/~jmorris/linux/cuecat/
    (map {
        tr/a-zA-Z0-9+-/ -_/;
        $_ = unpack 'u', chr(32 + length()*3/4) . $_;
        s/\0+$//;
        $_ ^= "C" x length;
    } (my $str = $_[0]) )[0];
}

1;

__DATA__
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 3.2 Final//EN">
<!-- $Id: BookQueue.pm,v 1.1 2004/12/06 07:12:19 jlh Exp $ -->
<html>
<head><title><TMPL_VAR NAME=BLOG_NAME> | BookQueue Movable Type Plugin</title></head>
<body onload="document.f.isbn.focus()">

<table border="0" cellpadding="10" cellspacing="0">
<tr><td>
<form name="f" method="get" action="<TMPL_VAR NAME=BQ_SCRIPT_URL>">
    <input type="hidden" name="__mode" value="save" />
    <input type="hidden" name="limit" value="<TMPL_VAR NAME=LIMIT>" />
    <input type="hidden" name="blog_id" value="<TMPL_VAR NAME=BLOG_ID>" />
    ISBN/EAN: <input name="isbn" type="text" size="11" />
</form>
</td>

<TMPL_IF NAME=REBUILD>
<td width="60%" align="center">
Changes saved. Click to <a title="Rebuild Files" onmouseover="window.status='Rebuild Files'; return true" href="#" onClick="window.open('mt.cgi?__mode=rebuild_confirm&blog_id=<TMPL_VAR NAME=BLOG_ID>', 'rebuild', 'width=400,height=200,resizable=yes')">REBUILD</a>.
</td>
</TMPL_IF>

</tr></table>

<table border="0" cellpadding="10" cellspacing="0">

<tr><td colspan="7"><hr /></td></tr>

<tr>
<td colspan="4">
show [ 

<TMPL_IF NAME=SHOW_ALL><b>all</b> | 
<TMPL_ELSE><a href="<TMPL_VAR NAME=BQ_SCRIPT_URL>?show=all&blog_id=<TMPL_VAR NAME=BLOG_ID>">all</a> | 
</TMPL_IF>

<TMPL_IF NAME=SHOW_UNREAD><b>unread</b> | 
<TMPL_ELSE><a href="<TMPL_VAR NAME=BQ_SCRIPT_URL>?show=unread&blog_id=<TMPL_VAR NAME=BLOG_ID>">unread</a> | 
</TMPL_IF>

<TMPL_IF NAME=SHOW_READING><b>reading</b> | 
<TMPL_ELSE><a href="<TMPL_VAR NAME=BQ_SCRIPT_URL>?show=reading&blog_id=<TMPL_VAR NAME=BLOG_ID>">reading</a> | 
</TMPL_IF>

<TMPL_IF NAME=SHOW_READ><b>read</b> 
<TMPL_ELSE><a href="<TMPL_VAR NAME=BQ_SCRIPT_URL>?show=read&blog_id=<TMPL_VAR NAME=BLOG_ID>">read</a> 
</TMPL_IF>

]
</td>

<TMPL_IF EMPTY>

</tr>

<tr><td align="center" bgcolor="#EEEEEE" colspan="7">no entries</td></tr>
<tr><td colspan="7"><hr /></td></tr>

<TMPL_ELSE>

<TMPL_IF PAGINATE>

<td colspan="3" align="right">
[ 

<TMPL_IF NAME=PREV><a href="<TMPL_VAR NAME=BQ_SCRIPT_URL>?show=<TMPL_VAR NAME=SHOW>&limit=<TMPL_VAR NAME=LIMIT>&offset=<TMPL_VAR NAME=PREV_OFFSET>&blog_id=<TMPL_VAR NAME=BLOG_ID>"><< prev <TMPL_VAR NAME=LIMIT></a>
<TMPL_ELSE><< prev <TMPL_VAR NAME=LIMIT> 
</TMPL_IF>
| 

<TMPL_IF NAME=NEXT><a href="<TMPL_VAR NAME=BQ_SCRIPT_URL>?show=<TMPL_VAR NAME=SHOW>&limit=<TMPL_VAR NAME=LIMIT>&offset=<TMPL_VAR NAME=NEXT_OFFSET>&blog_id=<TMPL_VAR NAME=BLOG_ID>">next <TMPL_VAR NAME=LIMIT> >></a> 
<TMPL_ELSE>next <TMPL_VAR NAME=LIMIT> >> 
</TMPL_IF>

]
</td>

</TMPL_IF>

</tr>

<TMPL_LOOP NAME=ENTRY_LOOP>
    <tr bgcolor="#<TMPL_IF NAME=ENTRY_ODD>FFFFFF<TMPL_ELSE>EEEEEE</TMPL_IF>">
    <form method="get" action="<TMPL_VAR NAME=BQ_SCRIPT_URL>">
        <input type="hidden" name="__mode" value="save" />
        <input type="hidden" name="show" value="<TMPL_VAR NAME=SHOW>" />
        <input type="hidden" name="limit" value="<TMPL_VAR NAME=LIMIT>" />
        <input type="hidden" name="offset" value="<TMPL_VAR NAME=OFFSET>" />
        <input type="hidden" name="blog_id" value="<TMPL_VAR NAME=BLOG_ID>" />
        <input type="hidden" name="key" value="<TMPL_VAR NAME=KEY>" />

        <td>
            <TMPL_VAR NAME=AMZN_INFO>
        </td>

        <td>
        ISBN:<br /><input type="text" name="isbn" size="11" value="<TMPL_VAR NAME=ISBN>" />
        </td>

        <td>
        <TMPL_LOOP NAME=STATUS_LOOP>
            <input type="radio" name="status" value="<TMPL_VAR NAME=STATUS>" <TMPL_IF NAME=CHECKED>checked</TMPL_IF> /><TMPL_VAR NAME=STATUS><br />
        </TMPL_LOOP>
        </td>

        <td>
        associated<br />entry id:<br /><input type="text" name="entry_id" size="6" value="<TMPL_VAR NAME=ENTRY_ID>" />
        </td>

        <td>
        last modified on:<br /><input type="text" name="modified_on" size="20" value="<TMPL_VAR NAME=MODIFIED_ON>" />
        <br /><input type="checkbox" name="mod_on_ovrd" <TMPL_IF NAME=MOD_ON_OVRD>checked</TMPL_IF> /> override
        </td>

        <td>
        delete<br /><input type="checkbox" name="delete" />
        </td>

        <td>
        <input type="submit" value="update" />
        </td>

    </form>
    </tr>
</TMPL_LOOP>

    <tr><td colspan="7"><hr /></td></tr>
    <tr><td colspan="7" align="right">
    <form method="get" action="<TMPL_VAR NAME=BQ_SCRIPT_URL>">
        <input type="hidden" name="__mode" value="view" />
        <input type="hidden" name="show" value="<TMPL_VAR NAME=SHOW>" />
        <input type="hidden" name="blog_id" value="<TMPL_VAR NAME=BLOG_ID>" />
        show 
        <select name="limit">
            <TMPL_LOOP NAME=LIMIT_LOOP>
                <option value="<TMPL_VAR NAME=LIMIT>"<TMPL_IF NAME=LIMIT_SELECTED> selected</TMPL_IF>><TMPL_VAR NAME=LIMIT_LABEL>
            </TMPL_LOOP>
        </select> 
        entries per page 
        <input type="submit" value="change" />
    </form>
    </td></tr>

</TMPL_IF>
</table>

</body>
</html>
