# BookQueue Movable Type Plugin
# http://jacob.hesch.cc/software/mt-plugins/BookQueue/
#
# $Id: bookqueue.pl,v 1.2 2004/12/06 07:59:01 jlh Exp $
#
# Copyright (C) 2003 Jacob L Hesch
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

package MTPlugins::BookQueue;
use strict;

use vars qw( $VERSION );
$VERSION = 1.01;

use MT::Template::Context;
use MT::PluginData;
use MT::Util qw( format_ts );

# NOTE: any mention here of an `entry_id' refers to a regular MT blog
# entry that may be associated with a BookQueue entry (BookQueue
# entries themselves do not have entry_id's)

MT::Template::Context->add_container_tag(BookQueueEntries => \&bq_entries);
MT::Template::Context->add_tag(BookQueueEntryISBN         => \&bq_entry_isbn);
MT::Template::Context->add_tag(BookQueueEntryStatus       => \&bq_entry_status);
MT::Template::Context->add_tag(BookQueueEntryDate         => \&bq_entry_date);
MT::Template::Context->add_tag(BookQueueEntryModDate      => \&bq_entry_mod_date);
MT::Template::Context->
    add_tag(BookQueueEntryBlogEntryID   => \&bq_entry_blog_entry_id);
MT::Template::Context->
    add_tag(BookQueueEntryBlogEntryLink => \&bq_entry_blog_entry_link);
MT::Template::Context->
    add_tag(BookQueueEntryBlogEntryPermalink => \&bq_entry_blog_entry_permalink);
MT::Template::Context->
    add_conditional_tag(BookQueueEntryIfBlogEntry => \&bq_entry_if_blog_entry);

sub bq_entries
{
    require MT::Entry;
    my ($ctx, $args) = @_;

    my $builder = $ctx->stash('builder');
    my $tokens  = $ctx->stash('tokens');
    my $blog_id = $ctx->stash('blog_id');

    my $lastn      = $args->{lastn}      || -1;
    my $offset     = $args->{offset}     || 0;
    my $status     = $args->{status}     || 'all';
    my $sort_order = $args->{sort_order} || 'descend';
    my $sort_by    = $args->{sort_by}    || 'entered_on';
    my $entry_id   = $args->{entry_id};
    my $isbn       = $args->{isbn};

    my $blog = MT::Blog->load($blog_id);
    $ctx->stash('bq_blog', $blog);

    # since the `blog_id', `status' and `modified_on' data fields of our
    # MTPluginData object are not real columns in the datastore, we
    # cannot use the `limit' and `offset' search parameters of MT::Object's
    # load() and load_iter() methods. so we must load *all* the plugin's
    # entries and do it ourselves...

    # since load_iter() sorts by `key' (same as `entered_on') by default,
    # if `$sort_by' is set to something other than `entered_on', push
    # all entries onto an array and do the sort and lastn/offset
    # limiting separately

    my $iter = MT::PluginData->load_iter({ plugin    => 'BookQueue' },
                                         { sort      => 'key',
                                           direction => $sort_order });
    my $res = '';
    my @entries;
    my $n_entries = my $i = 0;
    my @fields = qw( isbn status modified_on entry_id );

    ENTRY: while (my $entry = $iter->()) {
        next unless $blog_id == $entry->data->{blog_id};
        next unless $status eq 'all' or $status eq $entry->data->{status};
        if ($sort_by eq 'entry_id') {
            next unless $entry->data->{entry_id};
        }
        if ($entry_id) {
            next unless $entry->data->{entry_id}
            && $entry_id == $entry->data->{entry_id};
        }
        elsif ($isbn) {
            next unless $isbn == $entry->data->{isbn};
        }

        if ($sort_by eq 'entered_on') {
            next ENTRY unless ++$i > $offset;
            last ENTRY if $n_entries++ == $lastn;

            $ctx->stash('bq_entry_id', $entry->data->{entry_id});
            $ctx->stash('bq_date', $entry->key);
            for (@fields) {
                $ctx->stash('bq_' . $_, $entry->data->{$_} || '');
            }

            defined(my $out = $builder->build($ctx, $tokens))
                or return $ctx->error($ctx->errstr);
            $res .= $out;
        }
        else {
            push @entries, $entry;
        }
    }

    # since we are sorting on a data field that is not a real column
    # in the datastore, we could not use load_iter()'s sort feature...

    if ($sort_by ne 'entered_on') {
        if ($sort_order eq 'ascend') {
            @entries = sort {
                $a->data->{$sort_by} cmp $b->data->{$sort_by}
            } @entries;
        }
        else {
            @entries = sort {
                $b->data->{$sort_by} cmp $a->data->{$sort_by}
            } @entries;
        }
        for my $entry (@entries) {
            next unless ++$i > $offset;
            last if $n_entries++ == $lastn;

            $ctx->stash('bq_entry_id', $entry->data->{entry_id});
            $ctx->stash('bq_date', $entry->key);
            for (@fields) {
                $ctx->stash('bq_' . $_, $entry->data->{$_} || '');
            }

            defined(my $out = $builder->build($ctx, $tokens))
                or return $ctx->error($ctx->errstr);
            $res .= $out;
        }
    }

    $res;
}

sub bq_entry_isbn
{
    my $ctx = shift;
    $ctx->stash('bq_isbn');
}

sub bq_entry_status
{
    my $ctx = shift;
    $ctx->stash('bq_status');
}

sub bq_entry_date
{
    my ($ctx, $args) = @_;
    format_ts($args->{format}, $ctx->stash('bq_date'),
              $ctx->stash('bq_blog'));
}

sub bq_entry_mod_date
{
    my ($ctx, $args) = @_;
    format_ts($args->{format}, $ctx->stash('bq_modified_on'),
              $ctx->stash('bq_blog'));
}

sub bq_entry_if_blog_entry
{
    my $ctx = shift;
    $ctx->stash('bq_entry_id') ? 1 : 0;
}

sub bq_entry_blog_entry_id
{
    my $ctx = shift;
    my $entry_id = $ctx->stash('bq_entry_id')
	or return $ctx->error("No entry id found");
    $entry_id;
}

sub bq_entry_blog_entry_link
{
    my ($ctx, $args) = @_;
    my $archive_type = $args->{archive_type};
    my $entry_id     = $ctx->stash('bq_entry_id');
    my $entry        = MT::Entry->load($entry_id)
        or return $ctx->error("No entry found for id \"$entry_id\"");
    $entry->archive_url($archive_type ? $archive_type : ());
}

sub bq_entry_blog_entry_permalink
{
    my ($ctx, $args) = @_;
    my $archive_type = $args->{archive_type};
    my $entry_id     = $ctx->stash('bq_entry_id');
    my $entry        = MT::Entry->load($entry_id)
        or return $ctx->error("No entry found for id \"$entry_id\"");
    $entry->permalink($archive_type ? $archive_type : ());
}
